"""
Test stateful tokenizer for stream decoding.
"""
from transformers import AutoTokenizer

from basaran.tokenizer import StreamTokenizer


class TestTokenizer:
    """Test stateful tokenizer for stream decoding."""

    def test_byte_level_byte_pair_encoding(self):
        """Test with byte-level Byte-Pair-Encoding."""
        tokenizer = AutoTokenizer.from_pretrained(
            pretrained_model_name_or_path="./tests/data/tiny-random-bloom",
            local_files_only=True,
        )
        detokenizer = StreamTokenizer(tokenizer)

        expected = "hello world ABC \n 你好"
        tokens = tokenizer.encode(expected)

        actual = ""
        for token in tokens:
            actual += detokenizer.decode(token)

        assert actual == expected
        assert detokenizer.end == len(expected)

    def test_sentence_piece(self):
        """Test with SentencePiece."""
        tokenizer = AutoTokenizer.from_pretrained(
            pretrained_model_name_or_path="./tests/data/tiny-random-t5",
            local_files_only=True,
        )
        detokenizer = StreamTokenizer(tokenizer)

        expected = "hello world ABC"
        tokens = tokenizer.encode(expected)

        actual = ""
        for token in tokens:
            actual += detokenizer.decode(token)

        assert actual == expected
        assert detokenizer.end == len(expected)

    def test_llama_tokenizer(self):
        """Test with LLaMA tokenizer."""
        tokenizer = AutoTokenizer.from_pretrained(
            pretrained_model_name_or_path="./tests/data/tiny-random-llama",
            local_files_only=True,
        )
        detokenizer = StreamTokenizer(tokenizer)

        expected = "hello world ABC \n 你好"
        tokens = tokenizer.encode(expected)

        actual = ""
        for token in tokens:
            actual += detokenizer.decode(token)

        assert actual == expected
        assert detokenizer.end == len(expected)



"""
Use this script to download a model without loading it into memory.
This allows memory-constrained CI/CD runners to build container images
with large bundled models. See ../deployments/bundle/ for examples.
"""
import os
import sys
import tempfile

import huggingface_hub

if len(sys.argv) < 3:
    sys.exit("usage: python download.py REPO_ID LOCAL_DIR [REVISION]")

if os.getenv("TENSOR_FORMAT") == "safetensors":
    allow_patterns = ["*.safetensors", "*.safetensors.index.json"]
else:
    allow_patterns = ["*.bin", "*.bin.index.json"]

ignore_patterns = [
    ".*",
    "*.index.json",
    "*.bin",
    "*.ckpt",
    "*.h5",
    "*.mlmodel",
    "*.msgpack",
    "*.onnx",
    "*.ot",
    "*.pb",
    "*.safetensors",
    "*.tar.gz",
    "*.tflite",
]

kwargs = {
    "repo_id": sys.argv[1],
    "local_dir": sys.argv[2],
    "revision": sys.argv[3] if len(sys.argv) > 3 else None,
    "local_dir_use_symlinks": False,
    "resume_download": True,
}

with tempfile.TemporaryDirectory() as cache_dir:
    huggingface_hub.snapshot_download(
        cache_dir=cache_dir, ignore_patterns=ignore_patterns, **kwargs
    )
    huggingface_hub.snapshot_download(
        cache_dir=cache_dir, allow_patterns=allow_patterns, **kwargs
    )



"""
Test text generation model with stream decoding.
"""
from basaran.model import load_model


class TestModel:
    """Test text generation model with stream decoding."""

    def assert_stochastic(self, model):
        """Test completion using stochastic decoding."""
        prompt = "once upon a time"
        n = 2
        text = [""] * n
        finish_reasons = [None] * n
        for choice in model(
            prompt=prompt,
            min_tokens=10,
            max_tokens=16,
            temperature=0.95,
            top_p=0.95,
            n=n,
            logprobs=2,
            echo=True,
        ):
            index = choice["index"]
            text[index] += choice["text"]
            finish_reasons[index] = choice["finish_reason"]

        for i in range(n):
            if i > 0:
                assert text[i] != text[i - 1]
            assert prompt in text[i]
            assert finish_reasons[i] is not None

    def assert_deterministic(self, model):
        """Test completion using deterministic decoding."""
        n = 3
        text = [""] * n
        counts = [0] * n
        for choice in model(
            prompt=model.tokenize(""),
            min_tokens=10,
            max_tokens=16,
            temperature=0,
            n=n,
        ):
            index = choice["index"]
            text[index] += choice["text"]
            counts[index] += 1

        for i in range(n):
            if i > 0:
                assert text[i] == text[i - 1]
            assert counts[i] >= 10 and counts[i] <= 16


class TestDecoderOnlyModel(TestModel):
    """Test text generation using decoder-only models."""

    def test_stochastic(self):
        """Test completion using stochastic decoding."""
        model = load_model("./tests/data/tiny-random-bloom")
        self.assert_stochastic(model)

    def test_deterministic(self):
        """Test completion using deterministic decoding."""
        model = load_model("./tests/data/tiny-random-bloom")
        self.assert_deterministic(model)


class TestEncoderDecoderModel(TestModel):
    """Test text generation using encoder-decoder models."""

    def test_stochastic(self):
        """Test completion using stochastic decoding."""
        model = load_model("./tests/data/tiny-random-t5")
        self.assert_stochastic(model)

    def test_deterministic(self):
        """Test completion using deterministic decoding."""
        model = load_model("./tests/data/tiny-random-t5")
        self.assert_deterministic(model)


class TestLlamaModel(TestModel):
    """Test text generation using LLaMA models."""

    def test_stochastic(self):
        """Test completion using stochastic decoding."""
        model = load_model("./tests/data/tiny-random-llama")
        self.assert_stochastic(model)

    def test_deterministic(self):
        """Test completion using deterministic decoding."""
        model = load_model("./tests/data/tiny-random-llama")
        self.assert_deterministic(model)
